#!/bin/sh
# Author : Alexis Riksman, Matthijs van de Water, Edwin Mons
# Date   : 2006-07-28
# Update :
# Rev    : 0.3
#
# This shell script will start/stop/scan the WiFi interface on the eReader
#

# Some defaults
ethIf=wlan0
ethDrv=cf8385
ethDrv2=cfio

# Print usage message and quit
wirelessUsage()
{
    echo "Usage:"
    echo "   $0 stop"
    echo "   $0 start <ssid> [enc <wepkey>]"
    echo "   $0 scan [<ssid>]"
    echo "Note: always dhcp"
    exit 1
}

stopDHCPclient()
{
    if [ -e /var/run/udhcpc.$ethIf.pid ]
    then
        count=0
        pid=`cat /var/run/udhcpc.$ethIf.pid`
        kill ${pid}
        while kill -0 ${pid} 2>/dev/null
        do
            if [ ${count} -gt 5 ]
            then
                kill -9 ${pid}
            else
                sleep 1
                count=$((count+1))
            fi
        done
    fi
    rm -f /var/run/udhcpc.$ethIf.pid    # Just in case a crash left it there...
}

ifDown()
{
    ifconfig ${ethIf} down 2>/dev/null
    iwpriv ${ethIf} deepsleep 1
    rmmod ${ethDrv} 2>/dev/null
    rmmod ${ethDrv2} 2>/dev/null

    echo "-- Interface $ethIf down"
}

# MAIN #
########

# Take action
case "$1"
in
    "start")
        # Validate stuff
        ssid="$2"
        key="$4"
        [ -n "${ssid}" ] || wirelessUsage
        [ -z "$3" -o \( "$3" = "enc" -a -n "${key}" \) ] || wirelessUsage

        # Bring interface down in case it is (partly) active
        stopDHCPclient

        # Wake-up wireless controller, but keep interface disabled
        modprobe $ethDrv 2>/dev/null
        ##iwpriv $ethIf deepsleep 0

        # Set WEP encryption key
        if [ -n "${key}" ]
        then
            echo "encryption key: $key"
            iwconfig $ethIf key $key
            iwconfig $ethIf key on
        else
            iwconfig $ethIf key off
        fi

        # Connect to wireless network
        iwconfig $ethIf essid "$ssid" 
        # try again once if essid not found (workaround)
        if [ $? -eq 251 ]
        then
            iwconfig $ethIf essid "$ssid"
        fi
        if [ $? -ne 0 ] || ( iwconfig $ethIf|grep ESSID|grep -qv "ESSID:\"${ssid}\""; exit $? )
        then
            ifDown
            echo " -- The specified ESSID is not found"
            exit 1
        fi

        if udhcpc -n -p /var/run/udhcpc.$ethIf.pid -t 4 -i $ethIf
        then
            exit 0
        else
            ifDown
            exit 1
        fi
        ;;

    "stop")
        # Bring interface down
        stopDHCPclient
        ifDown
        exit 0
        ;;

    "scan")
        # wake-up wireless controller, but keep interface disabled
        modprobe $ethDrv           2>/dev/null
        ##iwpriv $ethIf deepsleep 0  > /dev/null
        ##ifconfig $ethIf down       > /dev/null

        if [ -n "$2" ]
        then
            iwconfig $ethIf essid "$2"
        fi

        # give wireless controller time to detect networks
        prev=0
        curr=1
        while [ $curr -gt $prev ]
        do
            scan_output=`iwlist $ethIf scan`
            prev=$curr
            curr=`echo "$scan_output" | wc -l`
        done

        # scan wireless networks and print one line per network
        echo "$scan_output" | \
            awk 'function print_network() {
                    if ( ssid != "" )
                    {
                        sec = "free"
                        if ( encr == 1 )
                        {
                            if ( wpa == 1 )
                            {
                                sec = "wpa"
                            }
                            else
                            {
                                sec = "wep"
                            }
                        }
                        #print "ssid=" ssid, "security=" sec, "quality=" qual
                        print "Network:", ssid, sec, qual
                    }
                 }

                 BEGIN {
                 }

                 $1 ~ /^Cell$/ {
                    print_network()
                    ssid = ""
                    encr = 0
                    qual = ""
                    wpa  = 0
                 }

                 $1 ~ /^ESSID:/ {
                     # ESSID:"ssid-string"
                     # note: ssid-string may contain white-space
                     i = index( $0, "ESSID:" )
                     ssid = substr( $0, i+6 )
                 }

                 $1 ~ /^Encryption$/ && $2 ~ /^key:/ {
                    # Encryption key: [on|off]
                    if ( substr( $2, 5 ) == "on" )
                    {
                        encr = 1
                    }
                 }

                 $1 ~ /^Quality:/ {
                    # Quality:x/y
                    qual = substr( $1, 9 )
                 }

                 $1 ~ /^Extra:wpa_ie=/ {
                    # Extra:wpa_ie=xxxxxxxxxxxxxxxxx
                    wpa=1
                 }

                 END {
                     print_network()
                 }'
        exit 0
        ;;

    *)
        wirelessUsage
        ;;
esac

exit 1  # code not reached

# vi:set ts=4 sw=4 ai et:
